package main

import (
	"bufio"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path"
)

func saveTeamFile(dirname string, team string, filename string, w http.ResponseWriter, r *http.Request) bool {
	if _, err := os.Stat(path.Join(dirname, team)); os.IsNotExist(err) || len(team) < 1 || team[0] == '_' {
		http.Error(w, "{\"errmsg\":\"Requête invalide.\"}", http.StatusBadRequest)
		return false
	} else {
		if len(filename) <= 0 {
			log.Println("EMPTY $EXERCICE RECEIVED:", filename)
			http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
			return false
		}

		// Previous submission not treated
		if _, err := os.Stat(path.Join(SubmissionDir, team, filename)); !os.IsNotExist(err) {
			http.Error(w, "{\"errmsg\":\"Du calme ! une requête est déjà en cours de traitement.\"}", http.StatusPaymentRequired)
			return false
		}

		if err := saveFile(path.Join(SubmissionDir, team), filename, r); err != nil {
			log.Println("Unable to handle submission file:", err)
			http.Error(w, "{\"errmsg\":\"Internal server error. Please retry in few seconds.\"}", http.StatusInternalServerError)
			return false
		}

		return true
	}
}

func saveFile(dirname string, filename string, r *http.Request) error {
	if _, err := os.Stat(dirname); os.IsNotExist(err) {
		if err := os.MkdirAll(dirname, 0777); err != nil {
			return err
		}
	}

	// Write content to temp file
	tmpfile, err := ioutil.TempFile(TmpSubmissionDir, "")
	if err != nil {
		return err
	}

	writer := bufio.NewWriter(tmpfile)
	reader := bufio.NewReader(r.Body)
	if _, err := reader.WriteTo(writer); err != nil {
		return err
	}
	writer.Flush()
	tmpfile.Close()

	if filename == "" {
		filename = path.Base(tmpfile.Name())
	}

	if err := os.Rename(tmpfile.Name(), path.Join(dirname, filename)); err != nil {
		log.Println("[ERROR] Unable to move file: ", err)
	}

	return nil
}
